  // module for public apis
  var Api = ( function (){
    // private functions
    // used for resetting the plugins lib
    var _emptyLib = function () {
        pluginsLibrary = {
          plugins: {},
          toCall: []
        }
        return this;
      },
      // setups new regular expressions for validating
      _newRegexp = function () {
        var args = [].slice.call(arguments);
      
        if( args.length >= 1 && typeof args[0] === 'string' ){
          regexpLibrary[args[0].toLowerCase().replace(/\s+/g, '')] = args[1];
          return this;
        }
        for( var n in args[0] ){
          regexpLibrary[n] = args[0][n];
        }
        return this;
      },
      // setups a new callable plugin. It will be fired 
      _newPlugin = function () {
        var args = [].slice.call(arguments);
      
        if( args.length >= 1 && typeof args[0] === 'string' ){
          if( !pluginsLibrary.plugins[args[0]] ) {
            pluginsLibrary.plugins[args[0]] = args[1];
            pluginsLibrary.toCall.push(args[0]);
          }
          return this;
        }
        for( var n in args[0] ){
          if( !pluginsLibrary.plugins[n] ) {
            pluginsLibrary.plugins[n] = args[0][n];
            pluginsLibrary.toCall.push(args[n]);
          }
        }
        return this;      
      },
      // sets a fixed width for tooltip
      _tooltipWidth = function(){
        var args = [].slice.call(arguments);
      
        args[1].width(args[2]);
        return this;
      };
    
    // public apis module
    var init = function (action, args) {
        this.action = action || '';
        this.args = args;
        return this.apiDispatcher();
      },
      // api actions library
      apiActions = {
        emptyLib: _emptyLib,
        newRegexp: _newRegexp,
        newPlugin: _newPlugin,
        tooltipWidth: _tooltipWidth
      },
      // responsible for dispatching the correct api action
      apiDispatcher = function () {
        if ( this.apiActions.hasOwnProperty( this.action ) ) {
          this.apiActions[this.action].apply(this, this.args);
        }
        return this;
      },
      // this function will call all the plugins in the plugin library
      callPlugins = function (helper) {
        
        if (!pluginsLibrary.toCall.length) {
          return this;
        }
        
        var k = 0, key;
        while( k < pluginsLibrary.toCall.length ) {
          key = pluginsLibrary.toCall[k];
          if ( typeof pluginsLibrary.plugins[key] == 'function' ) {
            pluginsLibrary.plugins[key].call(this, helper);
          }
          k++;
        }
        return this;
      };
    
    // prototyping
    init.prototype = {
      apiActions: apiActions,
      apiDispatcher: apiDispatcher,
      callPlugins: callPlugins
    }
      
    // returning api dispatching function
    return function (action, args) {
      return new init (action, args);
    }
  })();