  // tooltip module
  var Tooltip = (function ($, window) {
    
    // caching object
    var _cacheTooltip = {},
      // common tag for tooltips
      tooltipTagName = '<div/>',
      // main tooltip attributes
      tooltipAttributes = {
        'id':'senp_validate_tooltip',
        'class': 'senp-tooltip'
      },
      // tooltip header attributes
      tooltipHeaderAttributes = {
        'class': 'senp-tooltip-header'
      },
      // tooltip content attributes
      tooltipContentAttributes = {
        'class': 'senp-tooltip-content'
      };
    // private functions
    var _createTooltip = function(){
        this.tooltip = _cacheTooltip.tooltip = $(tooltipTagName, tooltipAttributes);
        this.tooltipHeader = _cacheTooltip.tooltipHeader = $(tooltipTagName, tooltipHeaderAttributes);
        this.tooltipContent = _cacheTooltip.tooltipContent = $(tooltipTagName, tooltipContentAttributes);
        // appending the new created elements
        this.tooltip.appendTo('body');
        this.tooltipHeader.appendTo(this.tooltip);
        this.tooltipContent.appendTo(this.tooltip);
      },
      _getElementPosition = function(el){
        
        var el = $(el),
          offset = el.offset();
        
        return {
          height: el.outerHeight(),
          left: offset.left,
          top: offset.top,
          width: el.outerWidth()
        }
      },
      // constructor object for the tooltip
      tooltip = function(){
        this.tooltip = _cacheTooltip.tooltip || '';
        this.tooltipHeader = _cacheTooltip.tooltipHeader || '';
        this.tooltipContent = _cacheTooltip.tooltipContent || '';
        this.createTooltip();
        this.target = 'body';
        return this;
      },
      // creates a tooltip if not exists
      createTooltip = function(){
        if(!this.tooltip.length){
          _createTooltip.call(this);
        }
        return this;
      },
      // gets the current validating element offset
      getElementPosition = function(el){
        this.elementPosition = _getElementPosition(el);
        return this;
      },
      guessTooltipPosition = function(){
        if (!this.elementPosition) {
          return this;
        }

        var windowInstance = $(window),
          returningObject = {};
        
        this.windowDims = {
          height: windowInstance.height(),
          width: windowInstance.width()
        }
        this.tooltipDims = {
          width: this.tooltip.outerWidth(),
          height: this.tooltip.outerHeight()
        }
        
        if (this.elementPosition.left > (windowInstance.width()/2)) {
          returningObject.left = this.elementPosition.left - this.elementPosition.width/2 - this.tooltipDims.width;
        }else{
          returningObject.left = this.elementPosition.left + this.elementPosition.width;
        }

        if (this.elementPosition.top > (windowInstance.height()/2)) {
          returningObject.top = this.elementPosition.top + this.elementPosition.height - this.tooltip.outerHeight();
        }else{
          returningObject.top = this.elementPosition.top;
        }
        return returningObject;
      },
      // sets title, message and footer content
      // to the tooltip
      setMessage = function(title, message){
        this.tooltipHeader.text(title);
        this.tooltipContent.html(message);
        return this;
      };
    // prototyping the constructor
    tooltip.prototype = {
      createTooltip: createTooltip,
      getElementPosition: getElementPosition,
      guessTooltipPosition: guessTooltipPosition,
      setMessage: setMessage
    }
    // returning as a module
    return function(){
      return new tooltip;
    }

  })($, window);//#end of tooltip module